"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderResultSummary = void 0;
const config_1 = require("../../config");
const types_1 = require("../types");
const tools_1 = require("../tools");
const log_1 = require("../../log");
const getResultType = (history) => {
  const last = history[history.length - 1];
  if (!last) return "nd"; // No data. Shouldn't happen
  switch (last.event) {
    case types_1.Event.Start:
      return "nc"; // Not completed. Shouldn't happen
    case types_1.Event.Success:
      return "ok";
    case types_1.Event.Failure:
      return "err";
  }
};
const calcDurationSec = (history) => {
  const len = history.length;
  if (!len) return 0;
  const first = history[0].timestamp;
  const last = history[len - 1].timestamp;
  return (last - first) / 1000;
};
const resultTypeWidth = 4;
const durationWidth = 5;
const durationFractionDigits = 2;
const resultWidth = resultTypeWidth + durationWidth + 2;
const renderUnitResult = (command, recorder) => {
  const history = recorder.getHistory(command);
  const resultType = getResultType(history);
  const duration = calcDurationSec(history);
  const { stdout } = process;
  stdout.write(resultType.padEnd(resultTypeWidth));
  const durationStr = String(duration.toFixed(durationFractionDigits));
  stdout.write(durationStr.padStart(durationWidth));
  stdout.write("s ");
};
const renderGroupResult = () => {
  const { stdout } = process;
  const resultType = "--";
  stdout.write(resultType);
  stdout.write(" ".repeat(resultWidth - resultType.length));
};
const renderCommandResult = (command, recorder) => {
  switch (command.type) {
    case types_1.CommandType.Unit:
      renderUnitResult(command, recorder);
      break;
    case types_1.CommandType.Group:
      renderGroupResult();
  }
};

/** Outputs result summary in a list form. */
const renderResultList = (topCommand, recorder) => {
  const { stdout } = process;
  tools_1.depthFirstSearch(topCommand, (command) => {
    switch (command.type) {
      case types_1.CommandType.Unit:
        renderUnitResult(command, recorder);
        stdout.write("| ");
        stdout.write(command.name);
        stdout.write("\n");
        break;
      case types_1.CommandType.Group:
        break;
    }
  });
};
const getDisplayName = (command) => {
  let prefix;
  switch (command.type) {
    case types_1.CommandType.Unit:
      prefix = "";
      break;
    case types_1.CommandType.Group:
      switch (command.subType) {
        case types_1.CommandSubType.Sequence:
          prefix = "[seq] ";
          break;
        case types_1.CommandSubType.Parallel:
          prefix = "[par] ";
          break;
        default:
          prefix = "[?] ";
          log_1.warn(
            `Command type mismatch: (${command.type}, ${command.subType}`
          );
          break;
      }
  }
  return prefix + command.name;
};

/** Outputs result summary in a tree form. */
const renderResultTree = (topCommand, recorder) => {
  const { stdout } = process;
  tools_1.depthFirstSearch(topCommand, (command, depth) => {
    renderCommandResult(command, recorder);
    stdout.write("| ");
    stdout.write("  ".repeat(depth));
    stdout.write(getDisplayName(command));
    stdout.write("\n");
  });
};

/** Writes result summary into standard output. */
const renderResultSummary = (topCommand, recorder) => {
  switch (config_1.resultSummaryType) {
    case config_1.ResultSummaryType.Tree:
      renderResultTree(topCommand, recorder);
      break;
    case config_1.ResultSummaryType.List:
      renderResultList(topCommand, recorder);
      break;
    case null:
      break;
    case undefined:
      break;
    default:
      log_1.warn(`Unknown result summary type: ${config_1.resultSummaryType}`);
  }
};
exports.renderResultSummary = renderResultSummary;
